@@ -1,8 +1,10 @@
-{-# OPTIONS_GHC -Wno-orphans #-}
+{-# LANGUAGE DataKinds #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE PolyKinds #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TypeApplications #-}
+{-# OPTIONS_GHC -Wno-orphans #-}
 
 module Witch.Instances where
 
@@ -23,6 +25,7 @@ import qualified Data.Map as Map
 import qualified Data.Ratio as Ratio
 import qualified Data.Sequence as Seq
 import qualified Data.Set as Set
+import qualified Data.Tagged as Tagged
 import qualified Data.Text as Text
 import qualified Data.Text.Encoding as Text
 import qualified Data.Text.Lazy as LazyText
@@ -201,11 +204,13 @@ instance TryFrom.TryFrom Int.Int32 Natural.Natural where
 -- | Uses 'fromIntegral' when the input is between -16,777,215 and 16,777,215
 -- inclusive.
 instance TryFrom.TryFrom Int.Int32 Float where
-  tryFrom = Utility.eitherTryFrom $ \s -> if s < -maxFloat
-    then Left Exception.Underflow
-    else if s > maxFloat
-      then Left Exception.Overflow
-      else Right $ fromIntegral s
+  tryFrom = Utility.eitherTryFrom $ \s ->
+    if s < -maxFloat
+      then Left Exception.Underflow
+      else
+        if s > maxFloat
+          then Left Exception.Overflow
+          else Right $ fromIntegral s
 
 -- | Uses 'fromIntegral'.
 instance From.From Int.Int32 Double where
@@ -264,20 +269,24 @@ instance TryFrom.TryFrom Int.Int64 Natural.Natural where
 -- | Uses 'fromIntegral' when the input is between -16,777,215 and 16,777,215
 -- inclusive.
 instance TryFrom.TryFrom Int.Int64 Float where
-  tryFrom = Utility.eitherTryFrom $ \s -> if s < -maxFloat
-    then Left Exception.Underflow
-    else if s > maxFloat
-      then Left Exception.Overflow
-      else Right $ fromIntegral s
+  tryFrom = Utility.eitherTryFrom $ \s ->
+    if s < -maxFloat
+      then Left Exception.Underflow
+      else
+        if s > maxFloat
+          then Left Exception.Overflow
+          else Right $ fromIntegral s
 
 -- | Uses 'fromIntegral' when the input is between -9,007,199,254,740,991 and
 -- 9,007,199,254,740,991 inclusive.
 instance TryFrom.TryFrom Int.Int64 Double where
-  tryFrom = Utility.eitherTryFrom $ \s -> if s < -maxDouble
-    then Left Exception.Underflow
-    else if s > maxDouble
-      then Left Exception.Overflow
-      else Right $ fromIntegral s
+  tryFrom = Utility.eitherTryFrom $ \s ->
+    if s < -maxDouble
+      then Left Exception.Underflow
+      else
+        if s > maxDouble
+          then Left Exception.Overflow
+          else Right $ fromIntegral s
 
 -- Int
 
@@ -328,11 +337,13 @@ instance TryFrom.TryFrom Int Natural.Natural where
 -- | Uses 'fromIntegral' when the input is between -16,777,215 and 16,777,215
 -- inclusive.
 instance TryFrom.TryFrom Int Float where
-  tryFrom = Utility.eitherTryFrom $ \s -> if s < -maxFloat
-    then Left Exception.Underflow
-    else if s > maxFloat
-      then Left Exception.Overflow
-      else Right $ fromIntegral s
+  tryFrom = Utility.eitherTryFrom $ \s ->
+    if s < -maxFloat
+      then Left Exception.Underflow
+      else
+        if s > maxFloat
+          then Left Exception.Overflow
+          else Right $ fromIntegral s
 
 -- | Uses 'fromIntegral' when the input is between -9,007,199,254,740,991 and
 -- 9,007,199,254,740,991 inclusive.
@@ -340,11 +351,13 @@ instance TryFrom.TryFrom Int Double where
   tryFrom = Utility.eitherTryFrom $ \s ->
     if toInteger (maxBound :: Int) <= maxDouble
       then Right $ fromIntegral s
-      else if s < -maxDouble
-        then Left Exception.Underflow
-        else if s > maxDouble
-          then Left Exception.Overflow
-          else Right $ fromIntegral s
+      else
+        if s < -maxDouble
+          then Left Exception.Underflow
+          else
+            if s > maxDouble
+              then Left Exception.Overflow
+              else Right $ fromIntegral s
 
 -- Integer
 
@@ -394,26 +407,30 @@ instance TryFrom.TryFrom Integer Natural.Natural where
   -- a bug in GHC 9.0.1. By inlining @fromNonNegativeIntegral@ and replacing
   -- @fromIntegral@ with @fromInteger@, we can work around the bug.
   -- https://mail.haskell.org/pipermail/haskell-cafe/2021-March/133540.html
-  tryFrom = Utility.eitherTryFrom
-    $ \s -> if s < 0 then Left Exception.Underflow else Right $ fromInteger s
+  tryFrom = Utility.eitherTryFrom $
+    \s -> if s < 0 then Left Exception.Underflow else Right $ fromInteger s
 
 -- | Uses 'fromIntegral' when the input is between -16,777,215 and 16,777,215
 -- inclusive.
 instance TryFrom.TryFrom Integer Float where
-  tryFrom = Utility.eitherTryFrom $ \s -> if s < -maxFloat
-    then Left Exception.Underflow
-    else if s > maxFloat
-      then Left Exception.Overflow
-      else Right $ fromIntegral s
+  tryFrom = Utility.eitherTryFrom $ \s ->
+    if s < -maxFloat
+      then Left Exception.Underflow
+      else
+        if s > maxFloat
+          then Left Exception.Overflow
+          else Right $ fromIntegral s
 
 -- | Uses 'fromIntegral' when the input is between -9,007,199,254,740,991 and
 -- 9,007,199,254,740,991 inclusive.
 instance TryFrom.TryFrom Integer Double where
-  tryFrom = Utility.eitherTryFrom $ \s -> if s < -maxDouble
-    then Left Exception.Underflow
-    else if s > maxDouble
-      then Left Exception.Overflow
-      else Right $ fromIntegral s
+  tryFrom = Utility.eitherTryFrom $ \s ->
+    if s < -maxDouble
+      then Left Exception.Underflow
+      else
+        if s > maxDouble
+          then Left Exception.Overflow
+          else Right $ fromIntegral s
 
 -- Word8
 
@@ -634,9 +651,10 @@ instance TryFrom.TryFrom Word.Word64 Float where
 -- | Uses 'fromIntegral' when the input is less than or equal to
 -- 9,007,199,254,740,991.
 instance TryFrom.TryFrom Word.Word64 Double where
-  tryFrom = Utility.eitherTryFrom $ \s -> if s <= maxDouble
-    then Right $ fromIntegral s
-    else Left Exception.Overflow
+  tryFrom = Utility.eitherTryFrom $ \s ->
+    if s <= maxDouble
+      then Right $ fromIntegral s
+      else Left Exception.Overflow
 
 -- Word
 
@@ -751,9 +769,10 @@ instance TryFrom.TryFrom Natural.Natural Float where
 -- | Uses 'fromIntegral' when the input is less than or equal to
 -- 9,007,199,254,740,991.
 instance TryFrom.TryFrom Natural.Natural Double where
-  tryFrom = Utility.eitherTryFrom $ \s -> if s <= maxDouble
-    then Right $ fromIntegral s
-    else Left Exception.Overflow
+  tryFrom = Utility.eitherTryFrom $ \s ->
+    if s <= maxDouble
+      then Right $ fromIntegral s
+      else Left Exception.Overflow
 
 -- Float
 
@@ -891,9 +910,10 @@ instance Integral a => From.From a (Ratio.Ratio a) where
 
 -- | Uses 'Ratio.numerator' when the denominator is 1.
 instance (Eq a, Num a) => TryFrom.TryFrom (Ratio.Ratio a) a where
-  tryFrom = Utility.eitherTryFrom $ \s -> if Ratio.denominator s == 1
-    then Right $ Ratio.numerator s
-    else Left Exception.LossOfPrecision
+  tryFrom = Utility.eitherTryFrom $ \s ->
+    if Ratio.denominator s == 1
+      then Right $ Ratio.numerator s
+      else Left Exception.LossOfPrecision
 
 -- | Uses 'fromRational'. This necessarily loses some precision.
 instance From.From Rational Float where
@@ -906,10 +926,9 @@ instance From.From Rational Double where
 -- | Uses `fromRational` as long as there isn't a loss of precision.
 instance Fixed.HasResolution a => TryFrom.TryFrom Rational (Fixed.Fixed a) where
   tryFrom = Utility.eitherTryFrom $ \s ->
-    let
-      t :: Fixed.Fixed a
-      t = fromRational s
-    in if toRational t == s then Right t else Left Exception.LossOfPrecision
+    let t :: Fixed.Fixed a
+        t = fromRational s
+     in if toRational t == s then Right t else Left Exception.LossOfPrecision
 
 -- Fixed
 
@@ -935,9 +954,10 @@ instance Num a => From.From a (Complex.Complex a) where
 
 -- | Uses 'Complex.realPart' when the imaginary part is 0.
 instance (Eq a, Num a) => TryFrom.TryFrom (Complex.Complex a) a where
-  tryFrom = Utility.eitherTryFrom $ \s -> if Complex.imagPart s == 0
-    then Right $ Complex.realPart s
-    else Left Exception.LossOfPrecision
+  tryFrom = Utility.eitherTryFrom $ \s ->
+    if Complex.imagPart s == 0
+      then Right $ Complex.realPart s
+      else Left Exception.LossOfPrecision
 
 -- NonEmpty
 
@@ -1023,19 +1043,37 @@ instance From.From ByteString.ByteString ShortByteString.ShortByteString where
 instance TryFrom.TryFrom ByteString.ByteString Text.Text where
   tryFrom = Utility.eitherTryFrom Text.decodeUtf8'
 
+-- | Uses 'Text.decodeUtf8''.
+instance TryFrom.TryFrom (Tagged.Tagged "UTF-8" ByteString.ByteString) Text.Text where
+  tryFrom = Utility.eitherTryFrom $ Text.decodeUtf8' . From.from
+
 -- | Converts via 'Text.Text'.
 instance TryFrom.TryFrom ByteString.ByteString LazyText.Text where
   tryFrom =
-    Utility.eitherTryFrom
-      $ fmap (Utility.into @LazyText.Text)
-      . Utility.tryInto @Text.Text
+    Utility.eitherTryFrom $
+      fmap (Utility.into @LazyText.Text)
+        . Utility.tryInto @Text.Text
+
+-- | Converts via 'Text.Text'.
+instance TryFrom.TryFrom (Tagged.Tagged "UTF-8" ByteString.ByteString) LazyText.Text where
+  tryFrom =
+    Utility.eitherTryFrom $
+      fmap (Utility.into @LazyText.Text)
+        . Utility.tryInto @Text.Text
 
 -- | Converts via 'Text.Text'.
 instance TryFrom.TryFrom ByteString.ByteString String where
   tryFrom =
-    Utility.eitherTryFrom
-      $ fmap (Utility.into @String)
-      . Utility.tryInto @Text.Text
+    Utility.eitherTryFrom $
+      fmap (Utility.into @String)
+        . Utility.tryInto @Text.Text
+
+-- | Converts via 'Text.Text'.
+instance TryFrom.TryFrom (Tagged.Tagged "UTF-8" ByteString.ByteString) String where
+  tryFrom =
+    Utility.eitherTryFrom $
+      fmap (Utility.into @String)
+        . Utility.tryInto @Text.Text
 
 -- LazyByteString
 
@@ -1055,19 +1093,37 @@ instance From.From LazyByteString.ByteString ByteString.ByteString where
 instance TryFrom.TryFrom LazyByteString.ByteString LazyText.Text where
   tryFrom = Utility.eitherTryFrom LazyText.decodeUtf8'
 
+-- | Uses 'LazyText.decodeUtf8''.
+instance TryFrom.TryFrom (Tagged.Tagged "UTF-8" LazyByteString.ByteString) LazyText.Text where
+  tryFrom = Utility.eitherTryFrom $ LazyText.decodeUtf8' . From.from
+
 -- | Converts via 'LazyText.Text'.
 instance TryFrom.TryFrom LazyByteString.ByteString Text.Text where
   tryFrom =
-    Utility.eitherTryFrom
-      $ fmap (Utility.into @Text.Text)
-      . Utility.tryInto @LazyText.Text
+    Utility.eitherTryFrom $
+      fmap (Utility.into @Text.Text)
+        . Utility.tryInto @LazyText.Text
+
+-- | Converts via 'LazyText.Text'.
+instance TryFrom.TryFrom (Tagged.Tagged "UTF-8" LazyByteString.ByteString) Text.Text where
+  tryFrom =
+    Utility.eitherTryFrom $
+      fmap (Utility.into @Text.Text)
+        . Utility.tryInto @LazyText.Text
 
 -- | Converts via 'LazyText.Text'.
 instance TryFrom.TryFrom LazyByteString.ByteString String where
   tryFrom =
-    Utility.eitherTryFrom
-      $ fmap (Utility.into @String)
-      . Utility.tryInto @LazyText.Text
+    Utility.eitherTryFrom $
+      fmap (Utility.into @String)
+        . Utility.tryInto @LazyText.Text
+
+-- | Converts via 'LazyText.Text'.
+instance TryFrom.TryFrom (Tagged.Tagged "UTF-8" LazyByteString.ByteString) String where
+  tryFrom =
+    Utility.eitherTryFrom $
+      fmap (Utility.into @String)
+        . Utility.tryInto @LazyText.Text
 
 -- ShortByteString
 
@@ -1093,10 +1149,18 @@ instance From.From Text.Text LazyText.Text where
 instance From.From Text.Text ByteString.ByteString where
   from = Text.encodeUtf8
 
+-- | Uses 'Text.encodeUtf8'.
+instance From.From Text.Text (Tagged.Tagged "UTF-8" ByteString.ByteString) where
+  from = From.from . Text.encodeUtf8
+
 -- | Converts via 'ByteString.ByteString'.
 instance From.From Text.Text LazyByteString.ByteString where
   from = Utility.via @ByteString.ByteString
 
+-- | Converts via 'ByteString.ByteString'.
+instance From.From Text.Text (Tagged.Tagged "UTF-8" LazyByteString.ByteString) where
+  from = fmap From.from . Utility.into @(Tagged.Tagged "UTF-8" ByteString.ByteString)
+
 -- LazyText
 
 -- | Uses 'LazyText.toStrict'.
@@ -1107,10 +1171,18 @@ instance From.From LazyText.Text Text.Text where
 instance From.From LazyText.Text LazyByteString.ByteString where
   from = LazyText.encodeUtf8
 
+-- | Uses 'LazyText.encodeUtf8'.
+instance From.From LazyText.Text (Tagged.Tagged "UTF-8" LazyByteString.ByteString) where
+  from = From.from . LazyText.encodeUtf8
+
 -- | Converts via 'LazyByteString.ByteString'.
 instance From.From LazyText.Text ByteString.ByteString where
   from = Utility.via @LazyByteString.ByteString
 
+-- | Converts via 'LazyByteString.ByteString'.
+instance From.From LazyText.Text (Tagged.Tagged "UTF-8" ByteString.ByteString) where
+  from = fmap From.from . Utility.into @(Tagged.Tagged "UTF-8" LazyByteString.ByteString)
+
 -- String
 
 -- | Uses 'Text.pack'. Some 'Char' values cannot be represented in 'Text.Text'
@@ -1135,16 +1207,25 @@ instance From.From LazyText.Text String where
 instance From.From String ByteString.ByteString where
   from = Utility.via @Text.Text
 
+-- | Converts via 'Text.Text'.
+instance From.From String (Tagged.Tagged "UTF-8" ByteString.ByteString) where
+  from = Utility.via @Text.Text
+
 -- | Converts via 'LazyText.Text'.
 instance From.From String LazyByteString.ByteString where
   from = Utility.via @LazyText.Text
 
+-- | Converts via 'LazyText.Text'.
+instance From.From String (Tagged.Tagged "UTF-8" LazyByteString.ByteString) where
+  from = Utility.via @LazyText.Text
+
 -- TryFromException
 
 -- | Uses @coerce@.
-instance From.From
-  (TryFromException.TryFromException source oldTarget)
-  (TryFromException.TryFromException source newTarget)
+instance
+  From.From
+    (TryFromException.TryFromException source oldTarget)
+    (TryFromException.TryFromException source newTarget)
 
 -- Day
 
@@ -1254,18 +1335,32 @@ instance From.From Time.NominalDiffTime Time.CalendarDiffTime where
 instance From.From Time.ZonedTime Time.UTCTime where
   from = Time.zonedTimeToUTC
 
+-- Tagged
+
+-- | Uses @coerce@. Essentially the same as 'Tagged.Tagged'.
+instance From.From a (Tagged.Tagged t a)
+
+-- | Uses @coerce@. Essentially the same as 'Tagged.unTagged'.
+instance From.From (Tagged.Tagged t a) a
+
+-- | Uses @coerce@. Essentially the same as 'Tagged.retag'.
+instance From.From (Tagged.Tagged t a) (Tagged.Tagged u a)
+
 --
 
-realFloatToRational
-  :: RealFloat s => s -> Either Exception.ArithException Rational
+realFloatToRational ::
+  RealFloat s => s -> Either Exception.ArithException Rational
 realFloatToRational s
   | isNaN s = Left Exception.LossOfPrecision
-  | isInfinite s = if s > 0
-    then Left Exception.Overflow
-    else Left Exception.Underflow
-  | otherwise = Right $ overPositive
-    (uncurry makeRational . uncurry fromDigits . Numeric.floatToDigits 10)
-    s
+  | isInfinite s =
+      if s > 0
+        then Left Exception.Overflow
+        else Left Exception.Underflow
+  | otherwise =
+      Right $
+        overPositive
+          (uncurry makeRational . uncurry fromDigits . Numeric.floatToDigits 10)
+          s
 
 overPositive :: (Eq a, Num a, Num b) => (a -> b) -> a -> b
 overPositive f x = if signum x == -1 then -(f (-x)) else f x
@@ -1277,8 +1372,8 @@ fromDigits ds e =
 makeRational :: Integer -> Integer -> Rational
 makeRational d e = toRational d * 10 ^^ e
 
-fromNonNegativeIntegral
-  :: (Integral s, Num t) => s -> Either Exception.ArithException t
+fromNonNegativeIntegral ::
+  (Integral s, Num t) => s -> Either Exception.ArithException t
 fromNonNegativeIntegral x =
   if x < 0 then Left Exception.Underflow else Right $ fromIntegral x
 
